

    
import argparse
import os
import os.path as osp
import time
import torch
from mmengine.config import Config, DictAction
from mmengine.hooks import Hook
from mmengine.logging import MMLogger
from mmengine.runner import Runner
from opencd.registry import RUNNERS

# ---------------------- GPU性能监控工具 ----------------------
import pynvml

# 初始化 NVML 库
pynvml.nvmlInit()
gpu_handle = pynvml.nvmlDeviceGetHandleByIndex(0)  # 假设使用第 0 块 GPU

# ---------------------- 参数解析 ----------------------
def parse_args():
    parser = argparse.ArgumentParser(description='Train a segmentor')
    parser.add_argument('config', help='train config file path')
    parser.add_argument('--work-dir', help='the dir to save logs and models')
    parser.add_argument(
        '--resume',
        action='store_true',
        default=False,
        help='resume from the latest checkpoint in the work_dir automatically')
    parser.add_argument(
        '--amp',
        action='store_true',
        default=False,
        help='enable automatic-mixed-precision training')
    parser.add_argument(
        '--cfg-options',
        nargs='+',
        action=DictAction,
        help='override some settings in the used config, the key-value pair '
        'in xxx=yyy format will be merged into config file. If the value to '
        'be overwritten is a list, it should be like key="[a,b]" or key=a,b '
        'It also allows nested list/tuple values, e.g. key="[(a,b),(c,d)]" '
        'Note that the quotation marks are necessary and that no white space '
        'is allowed.')
    parser.add_argument(
        '--launcher',
        choices=['none', 'pytorch', 'slurm', 'mpi'],
        default='none',
        help='job launcher')
    parser.add_argument('--local_rank', '--local-rank', type=int, default=0)
    args = parser.parse_args()
    if 'LOCAL_RANK' not in os.environ:
        os.environ['LOCAL_RANK'] = str(args.local_rank)
    return args

# ---------------------- 日志记录函数 ----------------------
def log_metrics(metrics, log_file):
    with open(log_file, 'a') as f:
        for key, value in metrics.items():
            f.write(f"{key}: {value}\n")
        f.write("\n")
    print(f"Metrics logged to {log_file}")
# ---------------------- 自定义日志 Hook ----------------------
class PerformanceLoggerHook(Hook):
    def __init__(self, work_dir, interval=10):
        """
        初始化自定义性能日志 Hook。
        :param work_dir: 日志文件保存的工作目录。
        :param interval: 记录间隔，以训练迭代数为单位。
        """
        self.work_dir = work_dir
        self.interval = interval  # 日志记录间隔
        self.max_power = 0

    def before_train_epoch(self, runner):
        # 初始化 epoch 开始时间
        runner.epoch_start_time = time.time()

    def before_train_iter(self, runner, batch_idx, data_batch=None):
        """
        在每次训练迭代前记录时间戳。
        :param runner: Runner 实例。
        :param batch_idx: 当前迭代的批次索引。
        :param data_batch: 当前批次的数据。
        """
        runner.step_start_time = time.time()  # 初始化 step_start_time

    def after_train_iter(self, runner, batch_idx, data_batch=None, outputs=None):
        """
        在每次训练迭代后记录性能指标。
        :param runner: Runner 实例。
        :param batch_idx: 当前迭代的批次索引。
        :param data_batch: 当前批次的数据。
        :param outputs: 当前迭代的输出结果（通常是 loss 等信息）。
        """
        # 按间隔记录
        if batch_idx % self.interval != 0:
            return
        # 确保 step_start_time 存在
        if not hasattr(runner, 'step_start_time'):
            runner.step_start_time = time.time()

        # 计算步时间和 GPU 状态
        step_time = time.time() - runner.step_start_time

        current_power = pynvml.nvmlDeviceGetPowerUsage(gpu_handle) / 1000.0  # 转换为瓦特
        self.max_power = max(self.max_power, current_power)

        # 显存使用和吞吐量
        memory_usage = torch.cuda.memory_allocated() / (1024 ** 3)  # 转换为 GB
        throughput = len(data_batch) / step_time if step_time > 0 else 0

        # 指标记录
        metrics = {
            'step_time': step_time,
            'throughput': throughput,
            'memory_usage': memory_usage,
            'max_power_consumption': self.max_power,
        }

        # 写入日志文件
        log_metrics(metrics, osp.join(self.work_dir, 'metrics.log'))

    def after_train_epoch(self, runner):
        """
        在每次训练 Epoch 后记录时间。
        :param runner: Runner 实例。
        """
        epoch_time = time.time() - runner.epoch_start_time
        metrics = {'epoch_time': epoch_time}
        log_metrics(metrics, osp.join(self.work_dir, 'metrics.log'))



# ---------------------- 主函数 ----------------------
def main():
    args = parse_args()
    cfg = Config.fromfile(args.config)

    # 确保 work_dir 存在
    work_dir = args.work_dir if args.work_dir else cfg.get('work_dir', './work_dir')
    cfg.work_dir = work_dir  # 添加到配置中
    os.makedirs(work_dir, exist_ok=True)

    # 实例化自定义 Hook
    performance_hook = PerformanceLoggerHook(work_dir=work_dir, interval=10)

   # 使用默认 Runner
    runner = Runner.from_cfg(cfg)
    runner.register_hook(performance_hook, priority='LOW')

    # 开始训练
    runner.train()


if __name__ == '__main__':
    main()
